"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var src_exports = {};
__export(src_exports, {
  createLRU: () => createLRU
});
module.exports = __toCommonJS(src_exports);
const createLRU = (options) => {
  let max = options.max;
  let head = null;
  let tail = null;
  let size = 0;
  let maxAgeCount = 0;
  const map = /* @__PURE__ */ new Map();
  const dispose = options == null ? void 0 : options.dispose;
  const maxAge = options == null ? void 0 : options.maxAge;
  const addNode = (node) => {
    node.next = head;
    if (head) head.prev = node;
    head = node;
    if (!tail) tail = node;
    size++;
  };
  const removeNode = (node) => {
    if (node.prev) node.prev.next = node.next;
    if (node.next) node.next.prev = node.prev;
    if (node === head) head = node.next;
    if (node === tail) tail = node.prev;
    size--;
  };
  const moveToHead = (node) => {
    removeNode(node);
    addNode(node);
  };
  function refresh(key) {
    var _a, _b;
    if (!(maxAge || maxAgeCount)) return true;
    const now = Date.now();
    if (key) {
      const node = map.get(key);
      if (!(node == null ? void 0 : node.timestamp)) return true;
      const effectiveMaxAge = (_a = node.maxAge) != null ? _a : maxAge;
      if (effectiveMaxAge && now - node.timestamp > effectiveMaxAge) {
        del(node.key);
        return false;
      }
      return true;
    }
    for (let current = tail; current !== null; current = current.prev) {
      if (!(current == null ? void 0 : current.timestamp)) continue;
      const effectiveMaxAge = (_b = current.maxAge) != null ? _b : maxAge;
      if (effectiveMaxAge && now - current.timestamp > effectiveMaxAge)
        del(current.key);
    }
  }
  const evict = (size2 = 1) => {
    for (let i = 0; i < size2; i++) {
      if (!tail) return;
      const tailKey = tail.key;
      const tailValue = tail.value;
      if (tail.maxAge && maxAgeCount > 0) {
        maxAgeCount--;
      }
      removeNode(tail);
      map.delete(tailKey);
      if (dispose) dispose(tailValue, tailKey);
    }
  };
  const set = (key, value, options2) => {
    let node = map.get(key);
    const now = maxAge || (options2 == null ? void 0 : options2.maxAge) ? Date.now() : void 0;
    if ((options2 == null ? void 0 : options2.maxAge) && !map.has(key)) maxAgeCount++;
    if (node) {
      node.value = value;
      node.timestamp = now;
      node.maxAge = options2 == null ? void 0 : options2.maxAge;
      if (head !== node) moveToHead(node);
      return;
    }
    node = {
      key,
      value,
      prev: null,
      next: null,
      timestamp: now
    };
    map.set(key, node);
    addNode(node);
    if (size > max) evict();
  };
  const get = (key) => {
    const node = map.get(key);
    if (!node) return;
    if (!refresh(key)) return;
    if (head !== node) moveToHead(node);
    return node.value;
  };
  const peek = (key) => {
    var _a;
    const node = map.get(key);
    if (!node) return;
    if (!refresh(key)) return;
    if (node == null ? void 0 : node.timestamp) {
      const now = Date.now();
      const effectiveMaxAge = (_a = node.maxAge) != null ? _a : maxAge;
      if (effectiveMaxAge && now - node.timestamp > effectiveMaxAge) {
        del(key);
        return;
      }
    }
    return node.value;
  };
  const has = (key) => {
    if (!refresh(key)) return false;
    return map.has(key);
  };
  const keys = () => {
    const result = [];
    for (let current = head; current !== null; current = current.next) {
      if (!refresh(current.key)) continue;
      result.push(current.key);
    }
    return result;
  };
  const values = () => {
    const result = [];
    for (let current = head; current !== null; current = current.next) {
      if (!refresh(current.key)) continue;
      result.push(current.value);
    }
    return result;
  };
  const entries = () => {
    const result = [];
    for (let current = head; current !== null; current = current.next) {
      if (!refresh(current.key)) continue;
      result.push([current.key, current.value]);
    }
    return result;
  };
  const forEach = (callback) => {
    for (let current = head; current !== null; current = current.next) {
      if (!refresh(current.key)) continue;
      callback(current.value, current.key);
    }
  };
  const del = (key) => {
    const node = map.get(key);
    if (!node) return false;
    if (node.maxAge && maxAgeCount > 0) {
      maxAgeCount--;
    }
    removeNode(node);
    map.delete(key);
    if (dispose) dispose(node.value, key);
    return true;
  };
  const clear = () => {
    if (dispose) for (const node of map.values()) dispose(node.value, node.key);
    map.clear();
    head = tail = null;
    size = 0;
    maxAgeCount = 0;
  };
  const resize = (newMax) => {
    max = newMax;
    for (let i = size; i > max; i--) evict();
  };
  const available = () => {
    refresh();
    return max - map.size;
  };
  const stored = () => {
    refresh();
    return map.size;
  };
  const maxSize = () => max;
  return {
    /** Adds a key-value pair to the cache. Updates the value if the key already exists. */
    set,
    /** Retrieves the value for a given key and moves the key to the most recent position. */
    get,
    /** Retrieves the value for a given key without changing its position in the cache. */
    peek,
    /** Checks if a key exists in the cache. */
    has,
    /** Returns an array of all keys in the cache, from most recent to least recent. */
    keys,
    /** Returns an array of all values in the cache, from most recent to least recent. */
    values,
    /** Returns an array of [key, value] pairs, from most recent to least recent. */
    entries,
    /** Iterates over each key-value pair in the cache, from most recent to least recent. */
    forEach,
    /** Deletes a key-value pair from the cache. */
    del,
    /** Evicts the oldest item or the specified number of the oldest items from the cache. */
    evict,
    /** Clears all key-value pairs from the cache. */
    clear,
    /** Resizes the cache to a new maximum size, evicting items if necessary. */
    resize,
    /** Returns the number of available slots in the cache before reaching the maximum size. */
    available,
    /** Returns the number of items currently stored in the cache. */
    stored,
    /** Returns the maximum number of items that can be stored in the cache. */
    maxSize
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createLRU
});
